@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# $Id: path.py,v 1.7 2002/02/03 18:48:47 doughellmann Exp $
+# $Id: path.py,v 1.9 2002/08/24 19:48:25 doughellmann Exp $
 #
 # Copyright 2001 Doug Hellmann.
 #
@@ -37,7 +37,7 @@ __rcs_info__ = {
     #  Creation Information
     #
     'module_name'  : '$RCSfile: path.py,v $',
-    'rcs_id'       : '$Id: path.py,v 1.7 2002/02/03 18:48:47 doughellmann Exp $',
+    'rcs_id'       : '$Id: path.py,v 1.9 2002/08/24 19:48:25 doughellmann Exp $',
     'creator'      : 'Doug Hellmann <DougHellmann@bigfoot.com>',
     'project'      : 'UNSPECIFIED',
     'created'      : 'Sat, 03-Feb-2001 12:49:56 EST',
@@ -46,8 +46,8 @@ __rcs_info__ = {
     #  Current Information
     #
     'author'       : '$Author: doughellmann $',
-    'version'      : '$Revision: 1.7 $',
-    'date'         : '$Date: 2002/02/03 18:48:47 $',
+    'version'      : '$Revision: 1.9 $',
+    'date'         : '$Date: 2002/08/24 19:48:25 $',
 }
 try:
     __version__ = __rcs_info__['version'].split(' ')[1]
@@ -57,15 +57,19 @@ except:
 #
 # Import system modules
 #
-import string
-import os
-import sys
 import glob
+import os
+import string
+import sys
+import unittest
 
 #
 # Import Local modules
 #
-from happydoclib.StreamFlushTest import StreamFlushTest
+from StreamFlushTest import StreamFlushTest
+from trace import trace as TRACE
+
+#TRACE.setVerbosity(3)
 
 #
 # Module
@@ -95,35 +99,55 @@ def rmkdir(path):
 
 def applyPrefixToPath(path, prefix):
     "Add the prefix value to every part of a given path."
+    TRACE.into('PATH', 'applyPrefixToPath', path=path, prefix=prefix)
+    drive, path = os.path.splitdrive(path)
     parts = string.split( path, os.sep )
-    #print 'PARTS:', parts
+    TRACE.writeVar(parts=parts)
     prefix_len = len(prefix)
     real_parts = []
     for p in parts:
         if not p:
             pass
         elif p not in ( '.', '..' ) and (p[:prefix_len] != prefix):
-            #print 'modifying "%s"' % p
+            TRACE.write('modifying "%s"' % p)
             p = '%s%s' % (prefix, p)
         real_parts.append(p)
 
-    #print 'REAL PARTS:', real_parts
+    TRACE.writeVar(real_parts=real_parts)
     name = apply(os.path.join, real_parts)
-    if path and path[0] == os.sep:
+    if drive:
+        name = os.sep.join((drive, name))
+    elif path and path[0] == os.sep:
         name = os.sep + name
+    TRACE.outof(name)
     return name
 
 def removePrefix(path, prefix):
     "Remove prefix from the beginning of path, if present."
+    TRACE.into('PATH', 'removePrefix', path=path, prefix=prefix)
+
+    path=os.path.normcase(path)
+    prefix=os.path.normcase(prefix)
+    TRACE.writeVar(path_after_norm_case=path)
+    TRACE.writeVar(prefix_after_norm_case=prefix)
+
     one_up = os.path.dirname(path)
+    TRACE.writeVar(one_up=one_up)
+    
     common_prefix = commonPrefix(one_up, prefix)
-    #print 'PATH: removePrefix( %s, %s )' % (path, prefix)
-    #print 'PATH:     common_prefix', common_prefix
+    TRACE.writeVar(common_prefix=common_prefix)
+    
     if common_prefix == prefix:
         path = path[len(common_prefix):]
+    else:
+        TRACE.write('common prefix (%s)' % common_prefix)
+        TRACE.write('does not match specified prefix (%s)' % prefix)
+        
+    TRACE.writeVar(pathMinusPrefix=path)
     while path and (path[0] == os.sep):
         path = path[1:]
-    #print 'PATH:     result', path
+        
+    TRACE.outof(path)
     return path
 
 def commonPrefix(path1, path2):
@@ -142,21 +166,40 @@ def commonPrefix(path1, path2):
     the path is considered the same only if the *full* directory or
     subdirectory name matches.
     """
-    #print 'commonPrefix(%s, %s)' % (path1, path2)
+    TRACE.into('PATH', 'commonPrefix', path1=path1, path2=path2)
     if not path1 or not path2:
+        TRACE.outof('')
         return ''
+    
+    drive1, path1 = os.path.splitdrive(path1)
+    TRACE.writeVar(drive1=drive1)
+    drive2, path2 = os.path.splitdrive(path2)
+    TRACE.writeVar(drive2=drive2)
+    if drive1 != drive2:
+        TRACE.outof('')
+        return ''
+    
     path1_parts = string.split(path1, os.sep)
+    TRACE.writeVar(path1_parts=path1_parts)
     path2_parts = string.split(path2, os.sep)
+    TRACE.writeVar(path2_parts=path2_parts)
+    
     common = []
     for p1, p2 in map(None, path1_parts, path2_parts):
-        if p1 == p2:
+        if p1 and p1 == p2:
+            TRACE.write('Found common part "%s"' % p1)
             common.append(p1)
+        elif not p1:
+            pass
         else:
             break
     common = string.join(common, os.sep)
-    if path1 and path1[0] == '/' and common and common[0] != '/':
-        common = '/' + common
-    #print '->"%s"' % common
+    if drive1 and common:
+        TRACE.write('have a drive spec')
+        common = os.path.normcase('%s%s%s' % (drive1, os.sep, common))
+    elif path1 and path1[0] == os.sep and common and common[0] != os.sep:
+        common = os.sep + common
+    TRACE.outof(common)
     return common
 
 def joinWithCommonMiddle(path1prefix, path1, path2):
@@ -184,22 +227,23 @@ def joinWithCommonMiddle(path1prefix, path1, path2):
     #
     # Find the part of path1 which is *not* part of path1prefix
     #
-    #print 'joinWithCommonMiddle(%s, %s, %s)' % (path1prefix, path1, path2)
+    TRACE.into('PATH', 'joinWithCommonMiddle', path1prefix=path1prefix,
+               path1=path1, path2=path2)
     common_prefix = commonPrefix(path1prefix, path1)
-    #print '  common prefix:', common_prefix
+    TRACE.writeVar(common_prefix=common_prefix)
     real_base = removePrefix(path1, common_prefix)
-    #print '  real base:', real_base
+    TRACE.writeVar(real_base=real_base)
     #
     # Remove the prefix common to the docset_real_base and
     # file_name.
     #
     common_prefix = commonPrefix(real_base, path2)
-    #print '  common prefix with real base and path2:', common_prefix
+    TRACE.write('common prefix with real base and path2:', common_prefix)
     path2 = removePrefix(path2, common_prefix)
-    #print '  fixed path2:', path2
+    TRACE.write('fixed path2:', path2)
             
     name = join( path1, path2 )
-    #print '->"%s"' % name
+    TRACE.outof(name)
     return name
     
 
@@ -220,41 +264,39 @@ def computeRelativeHTMLLink(fromName, toName, baseDirectory):
     and returns a string which, if used as the HREF in a link in
     fromName will point directly to toName.
     """
-    dbg=0
-    if dbg: print '\nPATH: FROM: ', fromName
-    if dbg: print 'PATH: TO  : ', toName
-    if dbg: print 'PATH: BASE: ', baseDirectory
-
+    TRACE.into('PATH', 'computeRelativeHTMLLink',
+               fromName=fromName,
+               toName=toName,
+               baseDirectory=baseDirectory,
+               )
     #
     # Normalize directory names
     #
     fromName = os.path.normpath(fromName)
     toName = os.path.normpath(toName)
-    if dbg: print 'PATH: FROM NORMALIZED : ', fromName
-    if dbg: print 'PATH: TO NORMALIZED   : ', toName
+    TRACE.writeVar(fromNameNormalized=fromName)
+    TRACE.writeVar(toNameNormalized=toName)
     
     #
     # Remove the base directory prefix from both
     #
     fromName = removePrefix(fromName, baseDirectory)
     toName = removePrefix(toName, baseDirectory)
-    if dbg: print 'PATH: FROM - PREFIX : ', fromName
-    if dbg: print 'PATH: TO   - PREFIX : ', toName
+    TRACE.writeVar(fromNameMinusPrefix=fromName)
+    TRACE.writeVar(toNameMinusPrefix=toName)
     
     if fromName == toName:
-        if dbg: print '\tPATH: same name'
+        TRACE.writeVar(toName=toName)
         relative_link = os.path.basename(toName)
     else:
         common_prefix = commonPrefix(os.path.dirname(fromName), os.path.dirname(toName))
         from_name_no_prefix = fromName[len(common_prefix):]
         while from_name_no_prefix and (from_name_no_prefix[0] == os.sep):
             from_name_no_prefix = from_name_no_prefix[1:]
-            if dbg: print '\tPATH: from, no prefix:', from_name_no_prefix
-            if dbg and from_name_no_prefix == 'z.html':
-                raise 'debug'
+            TRACE.writeVar(from_name_no_prefix=from_name_no_prefix)
         subdir_path = os.path.dirname(from_name_no_prefix)
         parts = string.split(subdir_path, os.sep)
-        if dbg: print '\tPATH: parts:', parts
+        TRACE.writeVar(parts=parts)
         if parts and parts[0]:
             levels = len(string.split(subdir_path, os.sep))
         else:
@@ -264,7 +306,7 @@ def computeRelativeHTMLLink(fromName, toName, baseDirectory):
         if to_name_no_prefix and (to_name_no_prefix[0] == os.sep):
             to_name_no_prefix = to_name_no_prefix[1:]
         relative_link = "%s%s" % (up_levels, to_name_no_prefix)
-    if dbg: print 'PATH: LINK: ', relative_link, '\n'
+    TRACE.outof(relative_link)
     return relative_link
 
 
@@ -303,7 +345,10 @@ def removeRelativePrefix(filename):
 ##
 def join( *args ):
     "os.path.join"
-    return apply(os.path.join, args)
+    TRACE.into('PATH', 'join', args=args)
+    result=apply(os.path.join, args)
+    TRACE.outof(result)
+    return result
 
 def cwd():
     "os.getcwd"
@@ -347,6 +392,15 @@ class PathTest(StreamFlushTest):
                'Path modification failed.\n\tExpected "%s",\n\tgot      "%s"' \
                % (expected, actual)
         return
+
+    if os.name == 'nt':
+        def testApplyPreifxToPathWin32(self):
+            expected = 'c:\\BLAH_tmp\\BLAH_foo'
+            actual = applyPrefixToPath('c:\\BLAH_tmp\\BLAH_foo', 'BLAH_')
+            assert actual == expected, \
+                   'Path modification failed.\n\tExpected "%s",\n\tgot      "%s"' \
+                   % (expected, actual)
+            return
     
     def testApplyPrefixToPathEmpty(self):
         expected = ''
@@ -371,6 +425,15 @@ class PathTest(StreamFlushTest):
                'Path modification failed.\n\tExpected "%s",\n\tgot      "%s"' \
                % (expected, actual)
         return
+
+    if os.name == 'nt':
+        def testRemovePrefixWin32(self):
+            expected = 'foo'
+            actual = removePrefix('c:\\tmp\\foo', 'c:\\tmp')
+            assert actual == expected, \
+                   'Path modification failed.\n\tExpected "%s",\n\tgot      "%s"' \
+                   % (expected, actual)
+            return
     
     def testRemovePrefixNotThere(self):
         expected = os.path.join('tmp', 'foo')
@@ -379,6 +442,15 @@ class PathTest(StreamFlushTest):
                'Path modification failed.\n\tExpected "%s",\n\tgot      "%s"' \
                % (expected, actual)
         return
+
+    if os.name == 'nt':    
+        def testRemovePrefixNotThereWin32(self):
+            expected = 'c:\\tmp\\foo'
+            actual = removePrefix('c:\\tmp\\foo', 'c:\\blah')
+            assert actual == expected, \
+                   'Path modification failed.\n\tExpected "%s",\n\tgot      "%s"' \
+                   % (expected, actual)
+            return
     
     def testRemovePrefixEmptyPath(self):
         expected = ''
@@ -405,6 +477,16 @@ class PathTest(StreamFlushTest):
                % (expected, actual)
         return
 
+    if os.name == 'nt':
+        def testCommonPrefixWin32(self):
+            expected = 'c:\\tmp'
+            actual = commonPrefix('c:\\tmp\\foo',
+                                  'c:\\tmp\\blah')
+            assert actual == expected, \
+                   'Path modification failed.\n\tExpected "%s",\n\tgot      "%s"' \
+                   % (expected, actual)
+            return
+
     def testCommonPrefixNone(self):
         expected = ''
         actual = commonPrefix(os.path.join('var', 'tmp', 'foo'),
@@ -415,6 +497,18 @@ class PathTest(StreamFlushTest):
                % (expected, actual)
         return
 
+    if os.name == 'nt':
+        def testCommonPrefixNoneWin32(self):
+            expected = ''
+            actual = commonPrefix('c:\\var\\tmp\\foo',
+                                  'c:\\tmp\\blah',
+                                  )
+            assert actual == expected, \
+                   'Path modification failed.\n\tExpected "%s",\n\tgot      "%s"' \
+                   % (expected, actual)
+            return
+
+
     def testCommonPrefixEmptyPaths(self):
         expected = ''
         actual = commonPrefix('', os.path.join('tmp', 'blah'))
@@ -438,6 +532,18 @@ class PathTest(StreamFlushTest):
                'Path modification failed.\n\tExpected "%s",\n\tgot      "%s"' \
                % (expected, actual)
         return
+
+    if os.name == 'nt':        
+        def testJoinWithCommonMiddleWin32(self):
+            expected = 'c:\\root\\one\\two\\three\\filename.txt'
+            actual = joinWithCommonMiddle('c:\\root\\one',
+                                          'c:\\root\\one\\two',
+                                          'two\\three\\filename.txt'
+                                          )
+            assert actual == expected, \
+                   'Path modification failed.\n\tExpected "%s",\n\tgot      "%s"' \
+                   % (expected, actual)
+            return
     
     def testJoinWithCommonMiddleNotCommon(self):
         expected = os.path.join('root', 'one', 'four', 'two', 'three', 'filename.txt')
@@ -449,6 +555,18 @@ class PathTest(StreamFlushTest):
                'Path modification failed.\n\tExpected "%s",\n\tgot      "%s"' \
                % (expected, actual)
         return
+
+    if os.name == 'nt':    
+        def testJoinWithCommonMiddleNotCommonWin32(self):
+            expected = 'c:\\%s' % os.path.join('root', 'one', 'four', 'two', 'three', 'filename.txt')
+            actual = joinWithCommonMiddle('c:\\%s' % os.path.join('root', 'one', 'five'),
+                                          'c:\\%s' % os.path.join('root', 'one', 'four'),
+                                          os.path.join('two', 'three', 'filename.txt')
+                                          )
+            assert actual == expected, \
+                   'Path modification failed.\n\tExpected "%s",\n\tgot      "%s"' \
+                   % (expected, actual)
+            return
     
     def testJoinWithCommonMiddleEmptyPrefix(self):
         expected = os.path.join('root', 'one', 'four', 'two', 'three', 'filename.txt')
@@ -460,6 +578,18 @@ class PathTest(StreamFlushTest):
                'Path modification failed.\n\tExpected "%s",\n\tgot      "%s"' \
                % (expected, actual)
         return
+
+    if os.name == 'nt':    
+        def testJoinWithCommonMiddleEmptyPrefixWin32(self):
+            expected = 'c:\\%s' % os.path.join('root', 'one', 'four', 'two', 'three', 'filename.txt')
+            actual = joinWithCommonMiddle('',
+                                          'c:\\%s' % os.path.join('root', 'one', 'four'),
+                                          os.path.join('two', 'three', 'filename.txt')
+                                          )
+            assert actual == expected, \
+                   'Path modification failed.\n\tExpected "%s",\n\tgot      "%s"' \
+                   % (expected, actual)
+            return
     
     def testJoinWithCommonMiddleEmptyPath1(self):
         expected = os.path.join('two', 'three', 'filename.txt')
@@ -539,3 +669,6 @@ class PathTest(StreamFlushTest):
                % (expected, actual)
         return
                
+
+if __name__ == '__main__':
+            unittest.main()
